# Here we give configuration in the form of a Ruby-syntax file built from Hashes.
# Other approaches to this include YAML (a simple format resembling Ruby syntax), 
# or a Domain Specific Language (see
# http://www.artima.com/rubycs/articles/ruby_as_dslP.html
# "Creating DSLs with Ruby", by Jim Freeze)

require "hamster_wars.rb"

# This callback method operates on the object which is the source
# of the transformation. Although the transformation code in general is 
# dynamic and does not "know" the types of the source and target objects,
# if you do know some fields of the object, you can use them -- 
# change them, log them, etc. 
# Return the source object or a replacement for it, to be used as source object.
def pre_transform(source)
	# As an example, charge the user $3 for passing the character from game to game.
	source.balance_dollars -= 3.00
	return source
end

# Operate on the object which is the target of the transformation 
# and return it or a replacement for it.
# See comment at pre_transform
def post_transform(target)
	# Log the full name of the customer in the transformed object.
	File.open("game_integration.log", "a") do |f|
		f.write("#{target.customer_full_name} ('#{target.ingame_name}') passed through\n")
	end
	return target
end


# This global constant holds the transformation-mappings between 
# a HamsterCharacter in Hamster Wars (the source) and a Persona in "Artichoke World".
#
# The $TRANSFORMATION variable holds a map where the key is a String
# and the value is another map.
$TRANSFORMATION =
{
   "HamsterWars::HamsterCharacter" , # Source class (in HamsterWars namespace)
     {
     	 # The colon indicates a Ruby "Symbol", a value useful as fixed keys in maps.
         :target => "Persona", # Target Class (default namespace)
         # In the mappings, the target_method is given (actually, an assignment method expressed with the equals sign)
         # followed by the expression to be calculated in the context of the source of the transformation.
         :mappings => { 
                       'customer_full_name', 'customer_last_name + ", "  + customer_first_name',
                       'ingame_name', 'character_alias',
                       'level', 'Math.sqrt(experience/1000).ceil',
                       'balance_euros', 'balance_dollars/1.25',
                       'health', '100 - hits'
                      }
    }
    
    # ... Add more class-to-class transformations here
}