require 'rexml/document'
include REXML

# Convert an XML file to an object. The XML
# file's name must correspond to the name of the Module of the class,
# the root element's name must correspond to the class' name,
# and the XML's root needs to hold a flat list
# of children elements, whose names correspond
# to a class' setter methods.
#
# Another approach would obviate the need for a matching class
# by generating the classs and attributes on-the-fly from the XML.
# See transform.rb for an example of how to do that.
def xmlfile_to_object(file_name)
	file = File.new(file_name)
	doc = Document.new(file)	
	doc_to_obj(file_name.split(".")[0], doc)
end

# Convert an XML file to an object. See xmlfile_to_object.
# The module name is provided as a parameter.
def xmlstring_to_object(module_name, xml_string)
	doc = Document.new(xml_string)
	doc_to_obj(module_name, doc)
end

# Assume a simple flat XML with classname in root 
# and scalar fields in flatly-nested sub-elements
def doc_to_obj(module_name, doc)
	# Dynamically load the relevant module.
	klass_module = Module.const_get(module_name)
	include klass_module
	root = doc.root
	begin
		# Dynamically load the relevant class.
	klass = Module.const_get(root.name)
		obj = klass.new
		root.elements.each() {
			|full_element|
			field_name = full_element.name
			content = full_element.text

			type = full_element.attributes['type']
			# As mentioned in the articles, Ruby is a strongly-
			# typed language -- every field must have a type.
			# All fields are treated here as String, unless
			# a type-code ('int') is given as an attribute 
			# in the XML. A more sophisticated approach 
			# would either support more XSI types given in 
			# attributes, or rely on an XSD for typing
			if type == 'int' then
				content = content.to_i
			end
			begin
				obj.send(field_name + "=", content)
			rescue NoMethodError => nme
				puts nme
			end
			
		}
	rescue NameError => ne
		puts "Cannot find class with name of XML root element #{root.name}"
	end
	
	return obj
	
end


def test_xml
	require "hamster_wars.rb"
	include HamsterWars
	obj = yield
	raise "nil object" unless obj
	raise "Hits are #{obj.hits} but expect 84" if obj.hits != 84
	raise obj.class.to_s unless obj.class.to_s == "HamsterWars::HamsterCharacter"
	puts "XML test succeeded"
end	


def test_from_file
	test_xml{
		 obj = xmlfile_to_object("HamsterWars.xml")
	}
end
 


def test_from_string
	test_xml{
		file = File.new("HamsterWars.xml") 
		string = file.readlines.join("\n")
		obj = xmlstring_to_object("HamsterWars", string)
	}
		
end


# If this file is executed directly, some simple 
# test methods will be run.
if $0 == __FILE__
	test_from_file
	test_from_string
end